# enable: C0103

class FooClass:
    """
    Just a class.
    """
    def __init__(self):
        """
        Some bad variable names.
        """
        FooBar = None
        Foo = None
        FOO = None
        foo_bar = None
