# enable: C0103

class FooBarBaz:
    """
    A good class name.
    """



class Foo:
    """
    A good class name.
    """

class _FooBarZar:
    """
    A class name begins with an underscore.
    """
