# enable: C0103

# Function names should be same as method names

def fooBar():
    pass


def foo():
    pass


def _foo():
    pass


def _fooBar():
    pass


def get_fooBar():
    pass


def _SOME_THING():
    pass


def test_SOME_THING():
    pass