# enable: C0103

MODULELEVELCONSTANT = True

class FooClass:
    """
    Just a class.
    """
    def __init__(self):
        """
        Some bad variable names.
        """
        f = None
        foo = None
        _foo = None
        fooBar = None
