# enable: C0103

class fooBarBaz:
    """
    Class name begins with lower-case letter.
    """



class boo:
    """
    Class name begins with lower-case letter.
    """



class Foo_Bar_Baz:
    """
    Class name contains underscore.
    """
